// (c) 2015, SAS Institute Inc.

jQuery.sap.require("sap.ui.core.routing.Router");
jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");

jQuery.sap.require("sas.risk.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.utils.NavChecker");

jQuery.sap.require("sas.risk.modimp.common.utils.CommonConstants");
jQuery.sap.require("sas.risk.modimp.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.modimp.common.utils.CommonAPI");
jQuery.sap.require("sas.risk.modimp.common.utils.MIPSession");
jQuery.sap.require("sas.risk.modimp.common.utils.PreferenceManager");
jQuery.sap.require("sas.risk.modimp.common.widgets.RiskBusyDialog");

jQuery.sap.require("sas.risk.modimp.implementation.models.RunInstance");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationAPI");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationConstants");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationUtil");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ViewManager");

jQuery.sap.require("sas.risk.modimp.inventory.utils.ModelInventoryConstants");


sap.ui.controller("sas.risk.modimp.implementation.views.ModelImplementation", function() {
    // pseudo-imports
    var CommonConstants = sas.risk.modimp.common.utils.CommonConstants;
    var CommonAPI = sas.risk.modimp.common.utils.CommonAPI;
    var MIPCommonUtil = sas.risk.modimp.common.utils.CommonUtil;
    var RiskCommonUtil = sas.risk.common.utils.CommonUtil;
    var MIPSession = sas.risk.modimp.common.utils.MIPSession;
    var ModelImplementationAPI = sas.risk.modimp.implementation.utils.ModelImplementationAPI;
    var ModelImplementationConstants = sas.risk.modimp.implementation.utils.ModelImplementationConstants;
    var ModelImplementationUtil = sas.risk.modimp.implementation.utils.ModelImplementationUtil;
    var ModelInventoryConstants = sas.risk.modimp.inventory.utils.ModelInventoryConstants;
    var PreferenceManager = sas.risk.modimp.common.utils.PreferenceManager;
    var ViewManager = sas.risk.modimp.implementation.utils.ViewManager;

    // vars
    var self;
    var router;
    var rb = MIPCommonUtil.getBundle();
    var navChecker;

    // run instance state polling
    var runInstancesStateInterval;
    var activeRunInstanceIds = [];


    /**
     * Manages loading of data for various selection controls on the ModelImplementationView.
     * @param {sap.ui.base.Event} evt  Event fired by the Router route matched listener
     * @private
     */
    function _init(evt) {
        var viewModel = new sas.risk.common.models.RiskJSONModel();
        self.getView().setModel(viewModel);

        var actionModel = new sas.risk.common.models.RiskJSONModel({
            canRunOrGenerate: false
        });
        self.getView().setModel(actionModel, "actions");

        _loadRunInstances(self.getRunInstancesUserToggleKey());

        // kick off state polling
        _initiateStatePolling();
    }

    /**
     * Loads the run instances and adds them to the view model.
     * @param {Boolean} currentUserOnly  if true, only instances created by the current user are loaded
     * @private
     */
    function _loadRunInstances(resultUserToggleKey) {
        var deferred = $.Deferred();
        var view = self.getView();
        // no key is current user
        var currentUserOnly = resultUserToggleKey !== ModelImplementationConstants.ALL_USERS_TOGGLE_KEY;

        view.setBusy(true);
        
        ModelImplementationAPI.loadRunInstances(currentUserOnly).done(
            function(runInstanceServerObjects, links) {
                try { // added for case 7612457847 FireFox workaround
                    var runInstances = [];
                    _.each(runInstanceServerObjects, function (runInstanceServerObject) {
                        runInstances.push(new sas.risk.modimp.implementation.models.RunInstance(runInstanceServerObject));
                    });

                    // applying sort here since I can't figure out how to get the table to set a default sort column
                    var sortedRunInstances = _.sortBy(runInstances, "creationTimeStamp").reverse();  // descending order

                    // determine which instances are active
                    activeRunInstanceIds = [];
                    _.each(sortedRunInstances, function (runInstance) {
                        if (_.contains(ModelImplementationConstants.ACTIVE_INSTANCE_STATE_VALUES, runInstance.stateProperties.state)) {
                            activeRunInstanceIds.push(runInstance.id);
                        }
                    });
																			
                    // can the current user run portfolio instances or generate code
                    var canRunOrGenerate = MIPCommonUtil.isCreateable(links, CommonConstants.REST_CREATE.rel);
                    view.getModel("actions").setProperty("/canRunOrGenerate", canRunOrGenerate);

                    // NOTE: setting this data can trigger a "too much recursion" error in FireFox per customer case 7612457847
                    // set the model on the instance table
                    view.getModel().setData(sortedRunInstances);

                } catch (err) {
                    console.log("v2 try-catch: " + err.message);
				    console.trace();
                }
                // refresh needs to know when this is done
                deferred.resolve();
            }
        ).fail(
            function(restError) {
                if (restError) {
                    MIPCommonUtil.showRESTErrorDialog(restError, rb.getText("errorDialog.title.loadRunInstances.txt"));
                }
            }
        ).always(function() {
            view.setBusy(false);
        });

        return deferred;
    }
	
    /**
     * Reloads the run instance having the specified ID and updates it in the view model.
     * @param {int} runInstanceId
     * @private
     */
    function _reloadRunInstance(runInstanceId) {
        ModelImplementationAPI.getRunInstance(runInstanceId).done(function(runInstanceServerObject) {
            var runInstancesModel = self.getView().getRunInstancesTable().getModel();
            var modelRunInstances = runInstancesModel.getData();
            var modelRunInstanceIndex = RiskCommonUtil.findIndex(modelRunInstances, function(modelRunInstance) {
                return modelRunInstance.id === runInstanceServerObject.id;
            });
            var runInstance = new sas.risk.modimp.implementation.models.RunInstance(runInstanceServerObject);
            runInstancesModel.setProperty("/" + modelRunInstanceIndex, runInstance);
            self.updateRunInstanceButtonEnabledState();
        });
    }

    /**
     * Initiates a poll that queries for the state of active run instances and updates
     * the model.
     * @param runInstances
     * @private
     */
    function _initiateStatePolling() {
        runInstancesStateInterval = setInterval(function() {
            // don't continue if there are no known active instances
            if(activeRunInstanceIds.length === 0) return;

            // Get the updated state for the currently active run instances
            ModelImplementationAPI.getUpdatedStates(activeRunInstanceIds).done(function(runInstanceStates) {
                // update the model with the returned states
                var completedRunInstanceIds = _updateRunInstanceStates(runInstanceStates);

                // if any instances completed, reload them so we can get the artifacts
                var runInstancesModel = self.getView().getRunInstancesTable().getModel();
                var modelRunInstances = runInstancesModel.getData();
                _.each(completedRunInstanceIds, function(completedRunInstanceId) {
                    _reloadRunInstance(completedRunInstanceId);
                });
            }).fail(function(restError) {
                // if restError doesn't contain details or items in the details, fail silently.
                if(!restError.details || !restError.details.items) {
                    return;
                }

                // remove failing instances from active run instances
                _.each(restError.details.items, function(item) {
                    activeRunInstanceIds = _.difference(activeRunInstanceIds, Number(item.id));
                });
            });
        }, 3000);   // run every 3 seconds (midtier updates on 3 second interval as well)
    }

    /**
     * Updates the run instances in the run instances table model corresponding to the specified
     * run instance state objects.
     * @param runInstanceStates
     * @return {Array}  the IDs of the run instances that have completed
     * @private
     */
    function _updateRunInstanceStates(runInstanceStates) {
        var runInstancesModel = self.getView().getModel();

        // update each run instance with the latest state information
        var completedInstanceIds = [];
        _.each(runInstanceStates, function(runInstanceState) {
            // find the run instance in the model
            var runInstanceIndex = RiskCommonUtil.findIndex(runInstancesModel.getProperty("/"), function(runInstance) {
                return runInstance.id === runInstanceState.id;
            });

            // S1227833
            // The run instance may have been filtered out by user toggle, so make
            // sure run instance is defined before proceeding, and if it isn't, remove
            // it from the active list.
            if(runInstanceIndex === -1) {
                activeRunInstanceIds = _.difference(activeRunInstanceIds, runInstanceState.id);
                return;     // may have filtered this out via user toggle
            }

            // we found the run, so extract it and update its state
            var runInstancePath = "/" + runInstanceIndex;
            var runInstance = runInstancesModel.getProperty(runInstancePath);
            var previousCompletedStepName = runInstance.getLastCompletedStepName();
            runInstancesModel.setProperty(runInstancePath + "/runTasks", runInstanceState.runTasks);
            runInstancesModel.setProperty(runInstancePath + "/stateProperties", runInstance.getStatePropertiesFromRunTasks(runInstanceState.runTasks));
            var currentCompletedStepName = runInstance.getLastCompletedStepName();

            // if a step completed since the last update, reload the run to get any new results
            if(previousCompletedStepName !== currentCompletedStepName) {
                _reloadRunInstance(runInstance.id);
            }

            // schedule completed instances for removal from active list
            if(!_.contains(ModelImplementationConstants.ACTIVE_INSTANCE_STATE_VALUES, runInstance.stateProperties.state)) {
                completedInstanceIds.push(runInstanceState.id);
            }

            // update button state and log the new state
            runInstance.canCancel = MIPCommonUtil.isDeleteable(runInstanceState.links, CommonConstants.REST_CANCEL.rel);
            self.updateRunInstanceButtonEnabledState();
            console.log("Updated state for run instance [" + runInstance.id + "]--new state is '" + runInstance.stateProperties.stateName + "'");
        });

        // remove completed instances from the active list
        activeRunInstanceIds = _.difference(activeRunInstanceIds, completedInstanceIds);

        return completedInstanceIds;
    }

    /**
     * Initiates the download of the entity in the browser.
     */
    function _downloadArtifact(artifact) {
        CommonAPI.downloadUrl(artifact.href, artifact.type).done(function (openWindowResult) {
            // nothing to do - the href downloaded successfully
        }).fail(function (restError) {  // restError=sas.risk.modimp.common.models.RESTError
            MIPCommonUtil.showRESTDownloadErrorDialog(restError, artifact.typeName);
        });
    }

    /**
     * Cancels the specified run instances.
     *
     * @param {Array} runInstances
     *
     * @private
     */
    function _cancelRunInstances(runInstances, busyDialog) {
        var view = self.getView();
        var runInstance = runInstances.shift();

        // lock down the view while we are canceling runs
        if(!busyDialog) {
            busyDialog = new sas.risk.modimp.common.widgets.RiskBusyDialog({
                title: rb.getText("ModelImplementation.CancelRunInstance.Busy.Canceling.fmt", runInstance.name),
                showCancelButton: false,
                openDelayMillis: 1500
            });
            busyDialog.open();
        }
        else {
            busyDialog.setTitle(rb.getText("ModelImplementation.CancelRunInstance.Busy.Canceling.fmt", runInstance.name));
        }

        var endCancelFunction = function() {
            view.getRunInstancesTable().clearSelection();
            _loadRunInstances(self.getRunInstancesUserToggleKey());
            if (busyDialog) {
                busyDialog.close();
            }
        };

        // cancel the run
        ModelImplementationAPI.cancelRunInstance(runInstance.id).done(
            function() {
                // if there are more runs left to cancel, call cancel again
                if(runInstances.length > 0) {
                    _cancelRunInstances(runInstances, busyDialog);
                }
                else {
                    endCancelFunction();
                }
            }
        ).fail(
            function(restError) {
                ModelImplementationAPI.getRunInstance(runInstance.id, false, false).done(
                    function(runInstanceServerObject) {
                        var runInstancesModel = view.getRunInstancesTable().getModel();
                        var modelRunInstances = runInstancesModel.getData();

                        var modelRunInstanceIndex = RiskCommonUtil.findIndex(modelRunInstances, function(modelRunInstance) {
                            return modelRunInstance.id === runInstanceServerObject.id;
                        });
                        var runInstance = new sas.risk.modimp.implementation.models.RunInstance(runInstanceServerObject);
                        runInstancesModel.setProperty("/" + modelRunInstanceIndex, runInstance);
                    }
                ).always(
                    function () {
                        var confirmFunction = function(action) {
                            // if user wishes to continue, call cancel again
                            if(action === sas.hc.m.MessageBox.Action.YES) {
                                _cancelRunInstances(runInstances, busyDialog);
                            }
                            else {
                                endCancelFunction();
                            }
                        };

                        // prevent busy dialog from being displayed while there is a confirmation dialog
                        busyDialog.close();
                        busyDialog = null;

                        // ignore mid-tier remediation here as it is not valid for the client
                        var clientRemediationOnly = true;

                        // If more runs to be canceled, notify the user of the error and confirm that
                        // they wish to proceed with remaining cancels. Otherwise just surface the error.
                        if(runInstances.length > 0) {
                            MIPCommonUtil.showErrorConfirmationDialog(
                                self.getView(),
                                rb.getText("errorDialog.title.cancelRunInstance.txt"),
                                restError.formatRESTErrorMessage(clientRemediationOnly),
                                rb.getText("ModelImplementation.ContinueCanceling.Confirm.txt"),
                                confirmFunction
                            );
                        }
                        else {
                            MIPCommonUtil.showRESTErrorDialog(restError, rb.getText("errorDialog.title.cancelRunInstance.txt"),
                                                                null, self.getView(), clientRemediationOnly);
                            endCancelFunction();
                        }
                    }
                );
            }
        );
    }

    /**
     * Deletes the specified run instances. If there is a failure at any point,
     * the user is notified and given the option to cancel deletion of the remaining
     * runs.
     *
     * @param {Array} runInstances
     * @param {Object} busyDialog
     * @param {Boolean} force is true when a forceful deletion is to be attempted
     *
     * @private
     */
    function _deleteRunInstances(runInstances, busyDialog, force) {
        var view = self.getView();
        var runInstance = runInstances.shift();

        // lock down the view while we are deleting runs
        if(!busyDialog) {
            busyDialog = new sas.risk.modimp.common.widgets.RiskBusyDialog({
                title: rb.getText("ModelImplementation.DeleteRunInstance.Busy.Deleting.fmt", runInstance.name),
                showCancelButton: false,
                openDelayMillis: 0
            });
            busyDialog.open();
        }
        else {
            busyDialog.setTitle(rb.getText("ModelImplementation.DeleteRunInstance.Busy.Deleting.fmt", runInstance.name));
        }

        var endDeleteFunction = function() {
            view.getRunInstancesTable().clearSelection();
            _loadRunInstances(self.getRunInstancesUserToggleKey());
            if (busyDialog) {
                busyDialog.close();
            }
        };

        // delete the run
        ModelImplementationAPI.deleteRunInstance(runInstance.id, force).done(
            function() {
                // if there are more runs left to delete, call delete again
                if(runInstances.length > 0) {
                    _deleteRunInstances(runInstances, busyDialog, force);
                }
                else {
                    endDeleteFunction();
                }
            }
        ).fail(
            function(restError) {
                ModelImplementationAPI.getRunInstance(runInstance.id, false, false).done(
                    function(runInstanceServerObject) {
                        var runInstancesModel = view.getRunInstancesTable().getModel();
                        var modelRunInstances = runInstancesModel.getData();

                        var modelRunInstanceIndex = RiskCommonUtil.findIndex(modelRunInstances, function(modelRunInstance) {
                            return modelRunInstance.id === runInstanceServerObject.id;
                        });
                        var runInstance = new sas.risk.modimp.implementation.models.RunInstance(runInstanceServerObject);
                        runInstancesModel.setProperty("/" + modelRunInstanceIndex, runInstance);
                    }
                ).always(
                    function () {
                        var confirmFunction = function(action) {
                            // if user wishes to continue, call delete again
                            if(action === sas.hc.m.MessageBox.Action.YES) {
                                _deleteRunInstances(runInstances, busyDialog, force);
                            }
                            else {
                                endDeleteFunction();
                            }
                        };

                        // prevent busy dialog from being displayed while there is a confirmation dialog
                        busyDialog.close();
                        busyDialog = null;

                        // If more runs to be deleted, notify the user of the error and confirm that
                        // they wish to proceed with remaining deletes. Otherwise just surface the error.
                        if(runInstances.length > 0) {
                            MIPCommonUtil.showErrorConfirmationDialog(
                                self.getView(),
                                rb.getText("errorDialog.title.deleteRunInstance.txt"),
                                restError.formatRESTErrorMessage(),
                                rb.getText("ModelImplementation.ContinueDeleting.Confirm.txt"),
                                confirmFunction
                            );
                        }
                        else {
                            // ignore mid-tier remediation when do a normal, non-force delete
                            var clientRemediationOnly = !force;
                            var response = rb.getText("errorDialog.title.deleteRunInstance.txt");

                            MIPCommonUtil.showRESTErrorDialog(restError, rb.getText("errorDialog.title.deleteRunInstance.txt"),
                                                                null, self.getView(), clientRemediationOnly);
                            endDeleteFunction();
                        }
                    }
                );
            }
        );
    }

    function _showModelingSystemsDialog(wizardLaunchFunction) {
        var rb = MIPCommonUtil.getBundle();
        var baseId = "sas.risk.modimp.implementation.views.ModelImplementation.SelectModelingSystem";

        var existingDialog = sap.ui.getCore().byId(baseId);
        if (existingDialog) {
            existingDialog.destroy();
        }

        var existingView = sap.ui.getCore().byId(ModelImplementationConstants.VIEW_SEARCH_MODELING_SYSTEMS);
        if (existingView) {
            existingView.destroy();
        }

        var modelingSystemView = ViewManager.loadSearchModelingSystems(true, false, false);
        var leftButton = new sas.hc.m.Button(baseId + '-leftButton',{
            text: rb.getText("button.ok.action.label"),
            enabled: "{actions>/okayActionEnabled}",
            press: function() {
                if (!!!selectModelingSystemDialog._okInProgress) {
                    selectModelingSystemDialog._okInProgress = true;
                    var modelingSystems = modelingSystemView.getSelectedModelingSystems();

                    if(modelingSystems && modelingSystems.length === 1) {
                        wizardLaunchFunction(modelingSystems[0].id, false);
                        selectModelingSystemDialog.close();
                    }
                    else {
                        MIPCommonUtil.showErrorConfirmationDialog(
                            self.getView(),
                            rb.getText('ModelImplementation.SelectModelingSystem.title.txt'),
                            rb.getText('ModelImplementation.SelectModelingSystem.NoModelingSystemSelected.txt')
                        );
                    }
                }
            }
        });
        leftButton.addStyleClass("sasPrimaryButton");
        var rightButton = new sas.hc.m.Button(baseId + '-rightButton',{
            text: rb.getText("button.cancel.action.label"),
            press: function () {
                selectModelingSystemDialog.close();
            }
        });

        var contentPanel =  sas.risk.common.widgets.WidgetFactory.buildPanelNoPadding({content: modelingSystemView});
        var selectModelingSystemDialog = new sas.hc.m.Dialog(baseId, {
            title: rb.getText('ModelImplementation.SelectModelingSystem.title.txt'),
            content: [ contentPanel ],
            contentHeight: "60%",
            contentWidth: "80%",
            leftButton: leftButton,
            rightButton: rightButton,
            afterClose: function () {
                selectModelingSystemDialog.destroy();
                selectModelingSystemDialog._okInProgress = false;
            }
        });
        selectModelingSystemDialog.addStyleClass("mipDialogPadding");
        selectModelingSystemDialog.setModel(modelingSystemView.getModel("actions"), "actions");
        selectModelingSystemDialog.open();
    }

    return {
        // lifecycle functions

        onInit: function () {
            self = this;

            router = sap.ui.core.routing.Router.getRouter(ModelImplementationConstants.ROUTER_ID);
            router.attachExactRouteMatched([ModelImplementationConstants.VIEW_MODEL_IMPLEMENTATION, ModelImplementationConstants.VIEW_DEFAULT], _init, this);

            var invRouter = sap.ui.core.routing.Router.getRouter(ModelInventoryConstants.ROUTER_ID);

            navChecker = new sas.risk.common.utils.NavChecker(
                CommonConstants.MIP_APPLICATION_ID,
                rb.getText("unsavedChanges.prompt.txt"),
                invRouter,
                router
            );
            navChecker.start($.proxy(this.getView().destroy, this.getView()));

            var table = self.getView().getRunInstancesTable();
            new sas.risk.common.utils.LayoutService(table,"RunInstancesTable");
        },

        onExit: function () {
            router.detachExactRouteMatched(ModelImplementationConstants.VIEW_MODEL_IMPLEMENTATION, _init, this);
            router.detachExactRouteMatched(ModelImplementationConstants.VIEW_DEFAULT, _init, this);

            //shut down the instance polling
            if(runInstancesStateInterval) {
                clearInterval(runInstancesStateInterval);
            }

            navChecker.stop();
        },

        getRouter: function() {
            return router;
        },

        getRunInstancesUserToggleKey: function() {
            var persistedKey = PreferenceManager.getRunInstancesUserToggleKey();

            if(!persistedKey) {
                return ModelImplementationConstants.CURRENT_USER_TOGGLE_KEY;    // default is current user
            }
            return persistedKey;
        },

        // button handlers

        handleLaunchCreatePortfolioCubeWizard: function(evt) {
            ViewManager.loadCreatePortfolioCubeWizard();
        },
        
        handleLaunchRunScenarioWizard: function(evt) {
            ViewManager.loadRunScenarioWizard();
        },

        handleLaunchRunScenarioWizardWithMSA: function(evt) {
            ViewManager.loadRunScenarioMSAWizard();
        },

        handleLaunchRunScenarioWizardWithBackTesting: function(evt) {
            ViewManager.loadRunScenarioBackTestingWizard();
        },

        handleLaunchRunEconomicSimulationWizard: function(evt) {
            ViewManager.loadRunEconomicSimulationWizard();
        },

        handleLaunchModelingSystemScenarioRun: function(evt) {
            _showModelingSystemsDialog($.proxy(ViewManager.loadRunScenarioWizardFromModelingSystem, ViewManager));
        },

        handleLaunchModelingSystemScenarioBackTestingRun: function(evt) {
            _showModelingSystemsDialog($.proxy(ViewManager.loadRunScenarioBackTestingWizardFromModelingSystem, ViewManager));
        },

        handleLaunchModelingSystemScenarioMSARun: function(evt) {
            _showModelingSystemsDialog($.proxy(ViewManager.loadRunScenarioMSAWizardFromModelingSystem, ViewManager));
        },

        handleLaunchModelingSystemSimulationRun: function(evt) {
            _showModelingSystemsDialog($.proxy(ViewManager.loadRunEconomicSimulationWizardFromModelingSystem, ViewManager));
        },

        /**
         * Loads either run instances for the current user or all users, depending on
         * the value of currentUserOnly.
         * @param {Boolean} currentUserOnly
         */
        handleRunInstancesFilterToggle: function(toggleKey) {
            PreferenceManager.saveResultUserToggleKey(toggleKey);
            _loadRunInstances(toggleKey);
            this.getView().getRunInstancesTable().clearSelection();
        },

        /**
         * Launches HP Risk to view the run instance Risk Cube results.
         */
        handleViewRunInstanceResults: function(cubeName, swcName, swcOutProperty, runInstanceLocation) {
            // tell Application Switcher to bring up HP Risk and send it the required data
            sas.applicationSwitcher.gotoApp(ModelImplementationConstants.HPRISK_APPLICATION_ID, ModelImplementationUtil.getHPRiskURL(), true);
            sas.applicationSwitcher.sendMessageToApp(
                ModelImplementationConstants.HPRISK_APPLICATION_ID,
                ModelImplementationConstants.HPRISK_MESSAGE_ID,
                {
                    cubeName: cubeName,
                    swcName: swcName,
                    swcProperty: swcOutProperty,
                    runInstance: runInstanceLocation,
                    workGroup: MIPSession.getCurrentWorkGroup().name === ModelImplementationConstants.MIP_PUBLIC_WORKGROUP_KEY ? ModelImplementationConstants.HPRISK_PUBLIC_WORKGROUP_KEY : MIPSession.getCurrentWorkGroup().name
                }
            );
        },

        /**
         * Launch the correct wizard in edit mode for the selected run.
         * @param {Object} selectedRunInstance
         */
        handleEditRunInstance: function(selectedRunInstance) {
            if(!selectedRunInstance) {
                return;
            }

            if(selectedRunInstance.type === ModelImplementationConstants.PORTFOLIO_CUBE_TYPE) {
                ViewManager.loadCreatePortfolioCubeWizardFromRun(selectedRunInstance.id, true);
            }
            else if(selectedRunInstance.type === ModelImplementationConstants.SCENARIO_TYPE) {
                ViewManager.loadRunScenarioWizardFromRun(selectedRunInstance.id,
                    true,
                    selectedRunInstance.modelingSystem ? selectedRunInstance.modelingSystem.id : null,
                    selectedRunInstance.msaConfigPresent,
                    selectedRunInstance.backTestingPresent
                );
            }
            else if(selectedRunInstance.type === ModelImplementationConstants.ECON_SIMULATION_TYPE) {
                ViewManager.loadRunEconomicSimulationWizardFromRun(selectedRunInstance.id, true, selectedRunInstance.modelingSystem ? selectedRunInstance.modelingSystem.id : null);
            }
        },

        /**
         * Populates the run portfolio fields based on the selected run instance.
         * @param {Object} selectedRunInstance
         */
        handleCopyRunInstance: function(selectedRunInstance) {
            if(selectedRunInstance.type === ModelImplementationConstants.PORTFOLIO_CUBE_TYPE) {
                ViewManager.loadCreatePortfolioCubeWizardFromRun(selectedRunInstance.id);
            }
            else if(selectedRunInstance.type === ModelImplementationConstants.SCENARIO_TYPE) {
                ViewManager.loadRunScenarioWizardFromRun(selectedRunInstance.id,
                    false,
                    selectedRunInstance.modelingSystem ? selectedRunInstance.modelingSystem.id : null,
                    selectedRunInstance.msaConfigPresent,
                    selectedRunInstance.backTestingPresent
                );
            }
            else if(selectedRunInstance.type === ModelImplementationConstants.ECON_SIMULATION_TYPE) {
                ViewManager.loadRunEconomicSimulationWizardFromRun(selectedRunInstance.id, false, selectedRunInstance.modelingSystem ? selectedRunInstance.modelingSystem.id : null);
            }
        },

        /**
         * Handler for the cancel run instance button. Cancels the currently selected
         * run instance.
         */
        handleCancelRunInstance: function() {
            _cancelRunInstances(self.getView().getSelectedRunInstances());
        },

        /**
         * Handler for the delete run instance button. Deletes the currently selected
         * run instance.
         */
        handleDeleteRunInstance: function() {
            _deleteRunInstances(self.getView().getSelectedRunInstances());
        },

        /**
         * Handler for the force delete run instance button. Forcefully deletes the currently selected
         * run instance.
         */
        handleForceDeleteRunInstance: function() {
            _deleteRunInstances(self.getView().getSelectedRunInstances(), null, true);
        },

        /**
         * Handler for the delete risk cubes button. Deletes the generated risk cube
         * for the currently selected run instance.
         */
        handleDeleteRiskCube: function() {
            var view = self.getView();
            var selectedRunInstance = view.getSelectedRunInstances()[0];  // only enabled if one instance is selected

            view.setRunInstanceTableBusy(true);
            ModelImplementationAPI.deleteRiskCube(selectedRunInstance.id).done(
                function() {
                    var runInstancesModel = view.getRunInstancesTable().getModel();
                    var modelRunInstances = runInstancesModel.getData();

                    // refresh the run instance
                    ModelImplementationAPI.getRunInstance(selectedRunInstance.id).done(function(runInstanceServerObject) {
                        var modelRunInstanceIndex = RiskCommonUtil.findIndex(modelRunInstances, function(modelRunInstance) {
                            return modelRunInstance.id === runInstanceServerObject.id;
                        });
                        var runInstance = new sas.risk.modimp.implementation.models.RunInstance(runInstanceServerObject);
                        runInstancesModel.setProperty("/" + modelRunInstanceIndex, runInstance);
                    });
                }
            ).fail(
                function(restError) {
                    MIPCommonUtil.showRESTErrorDialog(restError, rb.getText("errorDialog.title.deleteRiskCube.txt"));
                }
            ).always(
                function() {
                    view.getRunInstancesTable().clearSelection();
                    view.setRunInstanceTableBusy(false);
                }
            );
        },

        /**
         * Reloads the run instances.
         * @param {string} ModelImplementationConstants.ALL_USERS_TOGGLE_KEY for all runs. Otherwise, CURRENT_USER_TOGGLE_KEY
         * @param {boolean} true to suppress the refresh message indicator
         */
        handleRefreshButton: function(userToggleKey, suppressRefreshDisplay) {
            _loadRunInstances(userToggleKey).done(
                function () {
                    if (!suppressRefreshDisplay) {
                        sap.m.MessageToast.show(rb.getText("refresh.successful.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
                    }
                }
            );
        },

        /**
         * Trigger mid-tier to retrieve latest output result references from the filesystem for all selected run instances.
         * Then refresh the run instances. Assumes at least one selected run instance.
         */
        handleSyncOutput: function() {
            var view = self.getView();
            var runInstances = view.getSelectedRunInstances();
            var ids = _.pluck(runInstances, "id") || [];

            view.setRunInstanceTableBusy(true);
            ModelImplementationAPI.syncRunInstanceOutput(ids).done(
                function() {
                    self.handleRefreshButton(view.getActiveUserToggle(), true);
                    sap.m.MessageToast.show(rb.getText("ModelImplementation.synchronize.successful.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
                }
            ).fail(
                function(restError) {
                    MIPCommonUtil.showRESTErrorDialog(restError, rb.getText("errorDialog.title.syncRunInstanceOutput.txt"));
                }
            ).always(
                function() {
                    view.getRunInstancesTable().clearSelection();
                    view.setRunInstanceTableBusy(false);
                }
            );
        },

        /**
         * Launch a view for publishing the relevant contents of the selected run to
         * a modeling system.
         */
        handlePublishModelingSystem: function() {
            ViewManager.loadPublishModelingSystem(self.getView().getSelectedRunInstances()[0].id);
        },

        /**
         * Handles the search field event and passes the value to applyRunInstancesTableFilter.
         * @param {sap.ui.base.Event} e  The event triggered by liveChange on the search field
         * @private
         */
        handleSearchFilter: function(e) {
            this.applyRunInstancesTableFilter(e.getParameter("value") || e.getParameter("newValue") || e.getParameter("query") || "");
        },

        downloadArtifact: function(artifact) {
            _downloadArtifact(artifact);
        },

        /**
         * Filter the run instances on the table based on the input string.
         * @param {String} filterString  The string to filter on
         */
        applyRunInstancesTableFilter: function(filterString) {
            var table = this.getView().getRunInstancesTable();

            var allFilters = null;
            if(filterString.length > 0) {
                var columns = table.getColumns();
                var filters = [];

                // create filters for visible columns only
                _.each(columns, function (column) {
                    if (column.getVisible() && column.getFilterProperty() && column.getFilterOperator()) {
                        filters.push(new sap.ui.model.Filter(column.getFilterProperty(), column.getFilterOperator(), filterString));
                    }
                });

                allFilters = filters.length > 0 ? new sap.ui.model.Filter(filters) : null;
            }

            var binding = table.getBinding("rows");
            binding.filter(allFilters);
        },

        handleDisplayTransitionRatesDialog: function(artifact) {
            $.when(ModelImplementationAPI.getSortedListFromURI(artifact.uriForGetScenarios),
                ModelImplementationAPI.getSortedListFromURI(artifact.uriForGetTransitionMatrices)
            ).then(function() {
                    // The array of resolved objects as a pseudo-array
                    var scenarios = arguments[0];
                    var matrices = arguments[1];

                    // handle case of empty transbalance dataset
                    if (!matrices || 0 === matrices.length) {
                        MIPCommonUtil.showInformationDialog(
                            rb.getText("ModelImplementation.RunInstances.ViewTransbalanceRateDialog.title.txt"),
                            rb.getText("ModelImplementation.RunInstances.ViewTransbalanceRateDialog.empty.txt")
                        );
                        return;
                    }

                    var trcView = sas.risk.modimp.implementation.utils.ViewManager.loadTransitionRatesContent({
                        combinedMode: matrices && matrices.length > 0 && scenarios && scenarios.length > 0,
                        matrices: matrices,
                        scenarios: scenarios,
                        uriForGetTransitionRates: artifact.uriForGetTransitionRates,
                        uriForGetTransitionRatesComparison: artifact.uriForGetTransitionRatesComparison,
                        dialogContentReadyHandler: function() {
                            dialog.open();
                            trcView.setInitialFocus();
                        }
                    });
                    var trcController = trcView.getController();

                    var dialog = new sas.hc.m.Dialog({
                        title: rb.getText("ModelImplementation.RunInstances.ViewTransbalanceRateDialog.title.txt"),
                        content: trcView,
                        contentHeight: "100%",
                        contentWidth: "100%",
                        draggable: false,
                        rightButton: new sas.hc.m.Button({
                            text: rb.getText("button.close.action.label"),
                            press: function () {
                                dialog.close();
                            }
                        }),
                        afterClose: function () {
                            dialog.destroy();
                        }
                    });

                    dialog.addStyleClass("mipDialog");

                    self.getView().addDependent(dialog);

                }).fail(function(restError) {
                    if (restError) {
                        MIPCommonUtil.showRESTErrorDialog(restError, rb.getText("errorDialog.title.loadRunInstanceData.txt"));
                    }
                });
        },

        // state management

        /**
         * Determines which of the run instance actions should be enabled based on the
         * selected run instance.
         * @param evt (unused)
         */
        updateRunInstanceButtonEnabledState: function() {
            var enabledStates = {
                editInstanceEnabled: false,
                copyInstanceEnabled: false,
                deleteInstanceEnabled: false,
                publishModelingSystemEnabled: false,
                cancelRunEnabled: false,
                forceDeleteRunEnabled: false,
                deleteCubeEnabled: false,
                syncOutputEnabled: false
            };

            var runInstances = self.getView().getSelectedRunInstances();
            if(runInstances && runInstances.length > 0) {
                enabledStates.editInstanceEnabled = runInstances.length === 1 &&
                                                    runInstances[0].canEdit &&
                                                    _.contains(ModelImplementationConstants.INACTIVE_INSTANCE_STATE_VALUES, runInstances[0].getExecutionState());

                enabledStates.copyInstanceEnabled = runInstances.length === 1;

                enabledStates.deleteInstanceEnabled = _.every(runInstances, function(runInstance) {
                    return runInstance.canDelete && _.contains(ModelImplementationConstants.INACTIVE_INSTANCE_STATE_VALUES, runInstance.getExecutionState());
                });

                enabledStates.cancelRunEnabled = _.every(runInstances, function(runInstance) {
                    return runInstance.canCancel && _.contains(ModelImplementationConstants.ACTIVE_INSTANCE_STATE_VALUES, runInstance.getExecutionState());
                });

                enabledStates.forceDeleteRunEnabled = runInstances.length === 1 && runInstances[0].canDelete &&
                                                        _.contains(ModelImplementationConstants.INACTIVE_INSTANCE_STATE_VALUES, runInstances[0].getExecutionState());

                enabledStates.deleteCubeEnabled = runInstances.length === 1 && runInstances[0].canDelete && !!(runInstances[0].cubeDescriptor) && runInstances[0].type !== ModelImplementationConstants.PORTFOLIO_CUBE_TYPE;

                enabledStates.syncOutputEnabled = _.every(runInstances, function(runInstance) {
                    return _.contains(ModelImplementationConstants.INACTIVE_INSTANCE_STATE_VALUES, runInstance.getExecutionState());
                });

                enabledStates.publishModelingSystemEnabled = runInstances.length === 1 &&
                                                             runInstances[0].type !== ModelImplementationConstants.PORTFOLIO_CUBE_TYPE &&
                                                             runInstances[0].getExecutionState() === ModelImplementationConstants.COMPLETED_STATE;
            }

            self.getView().setRunInstanceButtonsEnabledStates(enabledStates);
        }

    };
}());